#*************************************************************
#
#                      puzzle.tcl
#                   ================
#                   Uwe Berger; 2010
#
# Ein kleines Bilderrtsel-Programm...
#
# Tastenbelegung:
# ---------------
# <Return> oder irgend eine Maustaste: deckt ein neues Feld auf
# <Escape>: deckt alle Felder auf
# <n>: naechstes Bild laden
# <q>: Programm beenden 
#
# Einige Parameter koennen am Anfang des Quelltextes :-) angepasst
# werden. (Wer Lust hat, kann ja noch eine Verarbeitung von Kommando-
# zeilenparametern implementieren, ... fuer meine Zwecke hat es
# so gereicht!)
#
# ---------
# Have fun!
#
#*************************************************************
package require tkpng

# Verzeichnis, in dem sich die PNG-Bilder befinden 
# ...(abschliessenden / nicht vergessen!)
set picture_path "bilder/"

# Anzahl der verdeckenden Felder in x- und y-Richtung
set cx 10
set cy 10

# Farbe der Abdeckfelder
set color green
set color_line black

# daraus noch ein paar interne Werte berechnen
set rec_count [expr $cx * $cy]
set rec_del_counter 0

#*************************************************************
proc delete_rec {} {
    global cx cy rec_count rec_del_counter
	if {$rec_del_counter < $rec_count} {
		set rx [expr {int($cx * rand())}]
		set ry [expr {int($cy * rand())}]
		while {[.c bbox rec_$ry$rx]=={}} {
			set rx [expr {int($cx * rand())}]
			set ry [expr {int($cy * rand())}]
		}
		.c delete rec_$ry$rx
		incr rec_del_counter
	}
}

#*************************************************************
proc delete_all_recs {} {
	global cx cy rec_count rec_del_counter
	for {set y 0} {$y < $cy} {incr y} {
		for {set x 0} {$x < $cx} {incr x} {
			.c delete rec_$y$x
		}
	}
	set rec_del_counter $rec_count
}

#*************************************************************
proc create_gui {} {
	wm resizable . 0 0
	canvas .c -height 10 -width 10
	pack .c
	# Bindings fuer Bedientasten 
	bind .c <ButtonPress> {delete_rec}
	bind . <Return> {delete_rec}
	bind . <Escape> {delete_all_recs}
	bind . <n> {set next_picture 1}
	bind . <q> {exit}
}

#*************************************************************
proc load_picture {picture} {
	global cx cy rec_del_counter color color_line
	# alles loeschen... 
	delete_all_recs
	.c delete picture
	set rec_del_counter 0
	# Foto laden, Groesse bestimmen, Canvasgroesse anpassen, Foto anzeigen
	set im [image create photo -file $picture]
	set p_dx [image width $im]
	set p_dy [image height $im]
	.c configure -height $p_dy -width $p_dx
	.c create image 0 0 -image $im -anchor nw -tag picture
	# Verdeckungsfelder generieren
	set dx [expr $p_dx / $cx]
	set dy [expr $p_dy / $cy]
	for {set y 0} {$y < $cy} {incr y} {
		for {set x 0} {$x < $cx} {incr x} {
			.c create rectangle [expr $x * $dx] [expr $y * $dy] [expr ($x + 1) * $dx] [expr ($y + 1)* $dy] \
								-fill $color -outline $color_line -tag rec_$y$x
		}
	}
}

#*************************************************************
proc shuffle {data} {
	set length [llength $data]
	for {set idx_1 0} {$idx_1 < $length} {incr idx_1} {
		set idx_2 [expr {int($length * rand())}]
		set temp [lindex $data $idx_1]
		lset data $idx_1 [lindex $data $idx_2]
		lset data $idx_2 $temp
	}
	return $data
}

#*************************************************************
#*************************************************************
#*************************************************************
# Verzeichnis nach *.png durchsuchen und in eine Liste
set dir [glob "$picture_path*.png"]
# List durchmischen
set dir [shuffle $dir]
# ein paa interne Variablen setzen
set picture_count [llength $dir]
set picture_idx 0
# GUI aufbauen
create_gui
# fuer jedes gefundene Bild Puzzle aufbauen und steuern
foreach {f} $dir {
	incr picture_idx 
	wm title . "Bilderrtsel ($picture_idx/$picture_count)" 
	load_picture $f
	vwait next_picture
	unset next_picture
}

